//=================================================================
//
//  Simple RF powermeter
//
//  (c) Dietmar Krause, DL2SBA
//
//  License see https://creativecommons.org/licenses/by/4.0/
//
#include <Nextion.h>
#include <NextionPage.h>
#include <NextionButton.h>
#include <NextionText.h>
#include <NextionProgressBar.h>
#include <NextionWaveform.h>
#include <NextionSlider.h>

#include <AD8318.h>

#include <PeakHold.h>
#include "config.h"
#include "data.h"

static NextionPage         pgMain(nex,  1, 0, "page0");
static NextionButton       bMode(nex,   1, 2, "b0");
static NextionButton       bFast(nex,   1, 8, "b1");
static NextionText         txtUnit(nex, 1, 1, "t0");
static NextionText         txtVal(nex,  1, 9, "t1");
static NextionText         txtPeakMin(nex, 1, 3, "t3");
static NextionText         txtPeakMax(nex, 1, 7, "t2");
static NextionProgressBar  pg0(nex,     1, 4, "pg0");
static NextionWaveform     wv0(nex,     1, 5, "s0");
static NextionSlider       sl0(nex,     1, 6, "h0");
static NextionText         txtOff (nex, 1, 10, "off");

AD8318 powerConverter = AD8318();

uint8_t convertTodBm = false;

extern void switchToSetupMode(NextionEventType type, INextionTouchable *widget);

//=================================================================
//
//  display speed text
//
static void displayModeText(void) {
  if ( fastMode ) {
    bFast.setText("slow");
  } else {
    bFast.setText("fast");
  }
}

//=================================================================
//
//  switch instrument into measurement mode
//
static void cbSpeed(NextionEventType type, INextionTouchable *widget) {
  fastMode =  (fastMode == 0 ) ? 1 : 0;
  displayModeText();
}

//=================================================================
//
//  switch instrument into measurement mode
//
static void cbToggleWatt(NextionEventType type, INextionTouchable *widget) {
  convertTodBm = !convertTodBm;
}

//=================================================================
//
//
void initRunMode ( void ) {
  bMode.attachCallback(&switchToSetupMode);
  bFast.attachCallback(&cbSpeed);
  txtUnit.attachCallback(&cbToggleWatt);
}



//=================================================================
//
// draw simple scale
//
void drawScale(void) {
  //draw major ticks
  for ( int i = 1; i < 6; ++i ) {
    int x = i * 53;
    nex.drawLine(x, 123, x, 110, 0x00000000);
    nex.drawLine(x + 1, 123, x + 1, 110, 0x00000000);
  }

  //  draw minor ticks
  for ( int i = 0; i < 6; ++i ) {
    int x = i * 53 + 26;
    nex.drawLine(x, 123, x, 115, 0x00000000);
  }
}

//=================================================================
//
//  switch instrument into measurement mode
//
void switchToRunMode(NextionEventType type, INextionTouchable *widget) {
  pgMain.show();

  // set backlight
  nex.setBrightness(backlight, false);

  //    cleanup the average buffer
  dataBufferIndex = 0;

  //  clear peak hold value
  txtPeakMin.setText("");
  txtPeakMax.setText("");

  //  get power offset in dB
  powerOffset_dB = powerSteps[powerOffset_Index];
  itoa(powerOffset_dB, dtostrfBuffer, 10); txtOff.setText(dtostrfBuffer);

  // setup peakHold
  minHold.setPeakHoldTime(peakHoldCurrent);
  maxHold.setPeakHoldTime(peakHoldCurrent);
  minHold.setDecayTime(peakDecayCurrent);
  maxHold.setDecayTime(peakDecayCurrent);

  // read the parameters for the AD8318 via PROGMEM interface
  powerConverter.readPROGMEM(kFactors + (calFreqIdx * powerConverter.getParmSize()));

  // set offset in converter
  powerConverter.setOffset(powerOffset_dB);

  // reset slider
  sl0.setValue(0);

  // redraw the slider scale
  drawScale();

  displayModeText();
  mode = MODE_RUN;
}

//=================================================================
//
//  do the stuff for measurement
//
void handleRunMode( void ) {
  static float vread;
  static float value;
  static uint8_t percent;
  static uint16_t localActValue;
  static uint16_t localAvgValue;
  static uint16_t localPeakValueMin;
  static uint16_t localPeakValueMax;
  static uint16_t localCurrentValue;

  // stop background task
  noInterrupts();

  //    copy isr values to local values
  localActValue = isrActValue;
  localAvgValue = isrAvgValue;
  localPeakValueMax = maxHold.getPeak();
  localPeakValueMin = minHold.getPeak();

  //    reenable background task
  interrupts();

  //  fastmode
  if ( fastMode ) {
    //  yes
    // use the current value
    localCurrentValue = localActValue;
  } else {
    //  no
    //  use the average value
    localCurrentValue = localAvgValue;
  }

  //  first do all dBm based
  //  convert value to dBm
  value = powerConverter.convert( localCurrentValue, true ) ;

  //  scale the dBm value in the range of the meter
  percent = 100 - ( ( 100 * ((value + powerOffset_dB) / (MIN_LOSS4BARS ) ) ) ) ;
  constrain (percent, 0, 99);

  // set the progress bar
  pg0.setValue(percent);

  // add to the diagram
  wv0.addValue(0, percent);

  // running in dBm mode?
  if ( convertTodBm ) {
    //  yes
    //  print the dBm
    dtostrf(value, 5, 2, dtostrfBuffer);  txtVal.setText(dtostrfBuffer);
    txtUnit.setText(powerConverter.getUnit());

    // convert Peak maximum to dBm and print
    value = powerConverter.convert(localPeakValueMax, true);
    dtostrf(value, 5, 2, dtostrfBuffer);    txtPeakMax.setText(dtostrfBuffer);

    // set slider
    percent =  101 - ( ( 100 * ((value + powerOffset_dB) / (MIN_LOSS4BARS ) ) ) ) ;
    constrain (percent, 0, 100);
    sl0.setValue(percent);

    // convert Peak minimum to dBm and print
    value = powerConverter.convert(localPeakValueMin, true);
    dtostrf(value, 5, 2, dtostrfBuffer);    txtPeakMin.setText(dtostrfBuffer);
  } else {
    //  convert to W
    value = powerConverter.convert( localCurrentValue, false ) ;

    //  print the W
    dtostrf(value, 5, 2, dtostrfBuffer);  txtVal.setText(dtostrfBuffer);
    txtUnit.setText(powerConverter.getUnit());

    //  running in Watt mode
    txtPeakMax.setText("");
    txtPeakMin.setText("");
    sl0.setValue(0);
  }
}

